@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 #    Copyright (C) 2014 Yahoo! Inc. All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,15 +14,13 @@
 
 import collections
 
-from debtcollector import removals
 import prettytable
-import six
 
 from automaton import _utils as utils
 from automaton import exceptions as excp
 
 
-class State(object):
+class State:
     """Container that defines needed components of a single state.
 
     Usage of this and the :meth:`~.FiniteMachine.build` make creating finite
@@ -33,12 +29,18 @@ class State(object):
     :ivar name: The name of the state.
     :ivar is_terminal: Whether this state is terminal (or not).
     :ivar next_states: Dictionary of 'event' -> 'next state name' (or none).
+    :ivar on_enter: callback that will be called when the state is entered.
+    :ivar on_exit: callback that will be called when the state is exited.
     """
 
-    def __init__(self, name, is_terminal=False, next_states=None):
+    def __init__(self, name,
+                 is_terminal=False, next_states=None,
+                 on_enter=None, on_exit=None):
         self.name = name
         self.is_terminal = bool(is_terminal)
         self.next_states = next_states
+        self.on_enter = on_enter
+        self.on_exit = on_exit
 
 
 def _convert_to_states(state_space):
@@ -51,12 +53,12 @@ def _convert_to_states(state_space):
 
 def _orderedkeys(data, sort=True):
     if sort:
-        return sorted(six.iterkeys(data))
+        return sorted(data)
     else:
-        return list(six.iterkeys(data))
+        return list(data)
 
 
-class _Jump(object):
+class _Jump:
     """A FSM transition tracks this data while jumping."""
     def __init__(self, name, on_enter, on_exit):
         self.name = name
@@ -64,7 +66,7 @@ class _Jump(object):
         self.on_exit = on_exit
 
 
-class FiniteMachine(object):
+class FiniteMachine:
     """A finite state machine.
 
     This state machine can be used to automatically run a given set of
@@ -98,31 +100,25 @@ class FiniteMachine(object):
         return cls.Effect(new_state['reactions'].get(event),
                           new_state["terminal"])
 
-    @removals.removed_kwarg('default_start_state',
-                            message="The usage of 'default_start_state' via"
-                            " the machine constructor is deprecated and will"
-                            " be removed in a future version; usage of"
-                            " the 'default_start_state' property setter is"
-                            " recommended.")
-    def __init__(self, default_start_state=None):
+    def __init__(self):
         self._transitions = {}
         self._states = collections.OrderedDict()
-        self._default_start_state = default_start_state
+        self._default_start_state = None
         self._current = None
         self.frozen = False
 
     @property
     def default_start_state(self):
-        return self._default_start_state
-
-    @default_start_state.setter
-    def default_start_state(self, state):
         """Sets the *default* start state that the machine should use.
 
         NOTE(harlowja): this will be used by ``initialize`` but only if that
         function is not given its own ``start_state`` that overrides this
         default.
         """
+        return self._default_start_state
+
+    @default_start_state.setter
+    def default_start_state(self, state):
         if self.frozen:
             raise excp.FrozenMachine()
         if state not in self._states:
@@ -141,10 +137,13 @@ class FiniteMachine(object):
         state_space = list(_convert_to_states(state_space))
         m = cls()
         for state in state_space:
-            m.add_state(state.name, terminal=state.is_terminal)
+            m.add_state(state.name,
+                        terminal=state.is_terminal,
+                        on_enter=state.on_enter,
+                        on_exit=state.on_exit)
         for state in state_space:
             if state.next_states:
-                for event, next_state in six.iteritems(state.next_states):
+                for event, next_state in state.next_states.items():
                     if isinstance(next_state, State):
                         next_state = next_state.name
                     m.add_transition(state.name, next_state, event)
@@ -178,10 +177,10 @@ class FiniteMachine(object):
         if state in self._states:
             raise excp.Duplicate("State '%s' already defined" % state)
         if on_enter is not None:
-            if not six.callable(on_enter):
+            if not callable(on_enter):
                 raise ValueError("On enter callback must be callable")
         if on_exit is not None:
-            if not six.callable(on_exit):
+            if not callable(on_exit):
                 raise ValueError("On exit callback must be callable")
         self._states[state] = {
             'terminal': bool(terminal),
@@ -225,7 +224,7 @@ class FiniteMachine(object):
         if state not in self._states:
             raise excp.NotFound("Can not add a reaction to event '%s' for an"
                                 " undefined state '%s'" % (event, state))
-        if not six.callable(reaction):
+        if not callable(reaction):
             raise ValueError("Reaction callback must be callable")
         if event not in self._states[state]['reactions']:
             self._states[state]['reactions'][event] = (reaction, args, kwargs)
@@ -358,11 +357,11 @@ class FiniteMachine(object):
         else:
             c.frozen = self.frozen
         if not shallow:
-            for state, data in six.iteritems(self._states):
+            for state, data in self._states.items():
                 copied_data = data.copy()
                 copied_data['reactions'] = copied_data['reactions'].copy()
                 c._states[state] = copied_data
-            for state, data in six.iteritems(self._transitions):
+            for state, data in self._transitions.items():
                 c._transitions[state] = data.copy()
         else:
             c._transitions = self._transitions
@@ -380,20 +379,20 @@ class FiniteMachine(object):
     @property
     def states(self):
         """Returns the state names."""
-        return list(six.iterkeys(self._states))
+        return list(self._states)
 
     @property
     def events(self):
         """Returns how many events exist."""
         c = 0
-        for state in six.iterkeys(self._states):
+        for state in self._states:
             c += len(self._transitions[state])
         return c
 
     def __iter__(self):
         """Iterates over (start, event, end) transition tuples."""
-        for state in six.iterkeys(self._states):
-            for event, target in six.iteritems(self._transitions[state]):
+        for state in self._states:
+            for event, target in self._transitions[state].items():
                 yield (state, event, target.name)
 
     def pformat(self, sort=True, empty='.'):
@@ -414,7 +413,7 @@ class FiniteMachine(object):
                 postfix_markings.append("^")
             if self._states[state]['terminal']:
                 postfix_markings.append("$")
-            pretty_state = "%s%s" % ("".join(prefix_markings), state)
+            pretty_state = "{}{}".format("".join(prefix_markings), state)
             if postfix_markings:
                 pretty_state += "[%s]" % "".join(postfix_markings)
             if self._transitions[state]:
@@ -453,9 +452,8 @@ class HierarchicalFiniteMachine(FiniteMachine):
     Effect = collections.namedtuple('Effect',
                                     'reaction,terminal,machine')
 
-    def __init__(self, default_start_state=None):
-        super(HierarchicalFiniteMachine, self).__init__(
-            default_start_state=default_start_state)
+    def __init__(self):
+        super().__init__()
         self._nested_machines = {}
 
     @classmethod
@@ -477,15 +475,15 @@ class HierarchicalFiniteMachine(FiniteMachine):
         if machine is not None and not isinstance(machine, FiniteMachine):
             raise ValueError(
                 "Nested state machines must themselves be state machines")
-        super(HierarchicalFiniteMachine, self).add_state(
+        super().add_state(
             state, terminal=terminal, on_enter=on_enter, on_exit=on_exit)
         if machine is not None:
             self._states[state]['machine'] = machine
             self._nested_machines[state] = machine
 
     def copy(self, shallow=False, unfreeze=False):
-        c = super(HierarchicalFiniteMachine, self).copy(shallow=shallow,
-                                                        unfreeze=unfreeze)
+        c = super().copy(shallow=shallow,
+                         unfreeze=unfreeze)
         if shallow:
             c._nested_machines = self._nested_machines
         else:
@@ -514,9 +512,9 @@ class HierarchicalFiniteMachine(FiniteMachine):
                                            also be used to initialize any state
                                            machines they contain (recursively).
         """
-        super(HierarchicalFiniteMachine, self).initialize(
+        super().initialize(
             start_state=start_state)
-        for data in six.itervalues(self._states):
+        for data in self._states.values():
             if 'machine' in data:
                 nested_machine = data['machine']
                 nested_start_state = None
