@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 #    Copyright (C) 2014 Yahoo! Inc. All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,6 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import functools
 import random
 
@@ -21,7 +20,6 @@ from automaton import exceptions as excp
 from automaton import machines
 from automaton import runners
 
-import six
 from testtools import testcase
 
 
@@ -41,7 +39,7 @@ class FSMTest(testcase.TestCase):
         return m
 
     def setUp(self):
-        super(FSMTest, self).setUp()
+        super().setUp()
         # NOTE(harlowja): this state machine will never stop if run() is used.
         self.jumper = self._create_fsm("down", add_states=['up', 'down'])
         self.jumper.add_transition('down', 'up', 'jump')
@@ -69,6 +67,40 @@ class FSMTest(testcase.TestCase):
         expected = [('down', 'jump', 'up'), ('up', 'fall', 'down')]
         self.assertEqual(expected, list(m))
 
+    def test_build_transitions_with_callbacks(self):
+        entered = collections.defaultdict(list)
+        exitted = collections.defaultdict(list)
+
+        def on_enter(state, event):
+            entered[state].append(event)
+
+        def on_exit(state, event):
+            exitted[state].append(event)
+
+        space = [
+            machines.State('down', is_terminal=False,
+                           next_states={'jump': 'up'},
+                           on_enter=on_enter, on_exit=on_exit),
+            machines.State('up', is_terminal=False,
+                           next_states={'fall': 'down'},
+                           on_enter=on_enter, on_exit=on_exit),
+        ]
+        m = machines.FiniteMachine.build(space)
+        m.default_start_state = 'down'
+        expected = [('down', 'jump', 'up'), ('up', 'fall', 'down')]
+        self.assertEqual(expected, list(m))
+
+        m.initialize()
+        m.process_event('jump')
+
+        self.assertEqual({'down': ['jump']}, dict(exitted))
+        self.assertEqual({'up': ['jump']}, dict(entered))
+
+        m.process_event('fall')
+
+        self.assertEqual({'down': ['jump'], 'up': ['fall']}, dict(exitted))
+        self.assertEqual({'up': ['jump'], 'down': ['fall']}, dict(entered))
+
     def test_build_transitions_dct(self):
         space = [
             {
@@ -229,13 +261,13 @@ class FSMTest(testcase.TestCase):
         self.assertFalse(self.jumper.terminated)
         self.assertEqual([('down', 'up'), ('up', 'down'), ('down', 'up')],
                          up_downs)
-        self.assertRaises(StopIteration, six.next, it)
+        self.assertRaises(StopIteration, next, it)
 
     def test_run_send_fail(self):
         up_downs = []
         runner = runners.FiniteRunner(self.jumper)
         it = runner.run_iter('jump')
-        up_downs.append(six.next(it))
+        up_downs.append(next(it))
         self.assertRaises(excp.NotFound, it.send, 'fail')
         it.close()
         self.assertEqual([('down', 'up')], up_downs)
@@ -331,7 +363,7 @@ class HFSMTest(FSMTest):
 
         def phone_reaction(old_state, new_state, event, chat_iter):
             try:
-                six.next(chat_iter)
+                next(chat_iter)
             except StopIteration:
                 return 'finish'
             else:
